/*
  ESP NTP user interface
  User configuration of NTP and Timezone definitions

  Targets for this sketch:
  - The user should be able to do his timezone definition with a listbox.
  - The user should be able to enter a NTP server.
  - The sketch should work without internet connection, meaning we want a local list of timezones.
  - A simple copy/paste of timezone brutto list to sketch.
  - Fallback in case of JavaScript is deactivated or fails.
  - Avoid the EEPROM emulation and persist data into preferences.

  This example is tested with 
  - ESP32 core 2.0.2
  - ESP8266 core 3.1.2

  https://werner.rothschopf.net/microcontroller/202402_arduino_esp_ntp_user_configuration.htm

  by noiasca 
  2024-02-11 
*/

//#include <credentials.h>                               // if you have a config file with your credentials

#ifndef STASSID
#define STASSID "your-ssid"                            // set your SSID
#define STAPSK  "your-password"                        // set your wifi password
#endif

/* *******************************************************************
         define which board variant should be compiled
         set the number to one of your defined boards
 ********************************************************************/

#define TXT_BOARDNAME "ESP NTP user interface"    // the name of the board on webgui and Serial

/* Configuration of NTP */
String ntpServer;
String tzshort; // timezone - see https://github.com/nayarsystems/posix_tz_db/blob/master/zones.csv
String tzlong;  // Area/Location

/* Necessary Includes */
#ifdef ARDUINO_ARCH_ESP32           // for the ESP32
#include <WiFi.h>                   // we need wifi to get internet access
#include <WebServer.h>              // for the webserver
#include <Preferences.h>            // part of ESP32 core
WebServer server(80);
#endif
#ifdef ARDUINO_ARCH_ESP8266         // for the ESP8266
#include <ESP8266WiFi.h>            // we need wifi to get internet access
#include <ESP8266WebServer.h>       // for the webserver
ESP8266WebServer server(80);        // an instance for the webserver
#include <Preferences.h>            // by Volodymyr Shymanskyy - install via library manager
#endif
#include <time.h>                   // time() ctime() for NTP
Preferences preferences;
#include "ntp.h"
#include "server.h"                 // the webserver with the user interface

void setup() {
  Serial.begin(115200);
  delay(200); // just give Serial some time to start
  Serial.println(TXT_BOARDNAME);

  preferences.begin("config");  // prepare preferences
  // prepare NTP & TZ
  ntpServer = preferences.getString("ntpServer", "pool.ntp.org"); // second parameter is default value in case there is no data in preferences
  tzshort = preferences.getString("tzshort", "CET-1CEST,M3.5.0/02,M10.5.0/03");
  tzlong = preferences.getString("tzlong", "Europe/Vienna");
  Serial.print(F("ntpServer=")); Serial.println(ntpServer);
  Serial.print(F("tzshort="));   Serial.println(tzshort);
  char ntpServerCA[42];          // we need the data in character arrays
  char tzshortCA[42];
  ntpServer.toCharArray(ntpServerCA, sizeof(ntpServerCA));
  tzshort.toCharArray(tzshortCA, sizeof(tzshortCA));
  
#ifdef ARDUINO_ARCH_ESP32             // for the ESP32 (needs 3 lines)
  configTime(0, 0, ntpServerCA);      // 0, 0 because we will use TZ in the next line
  setenv("TZ", tzshortCA, 1);         // Set environment variable with your time zone
  tzset();
#endif
#ifdef ARDUINO_ARCH_ESP8266           // for the ESP8266
  configTime(tzshortCA, ntpServerCA); // the ESP8266 you only need this IMPORTANT ONE LINER in your sketch!
#endif
  // remark: by default, the NTP will be started after 60 secs
  
  // start network
  WiFi.persistent(false);
  WiFi.mode(WIFI_STA);
  WiFi.begin(STASSID, STAPSK);
  while (WiFi.status() != WL_CONNECTED) {
    delay(200);
    Serial.print('.');
  }
  Serial.println(F("\nWiFi connected"));
  Serial.print(F("IP address: ")); Serial.println(WiFi.localIP());
  serverSetup();   // defines the resources and starts the webserver
}

void loop() {
  server.handleClient();               // call the webserver
  showTime();
}
//
